<?php

/**
 * Server layer over the OAuthRequest handler
 * 
 * @version $Id: OAuthServer.php 5 2008-02-13 12:29:12Z marcw@pobox.com $
 * @author Marc Worrell <marc@mediamatic.nl>
 * @copyright (c) 2007 Mediamatic Lab
 * @date  Nov 27, 2007 12:36:38 PM
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

require_once 'oauth_request_verifier.php';

class OAuthServer extends OAuthRequestVerifier
{
	/**
	 * Handle the request_token request.
	 * Returns the new request token and request token secret.
	 * 
	 * TODO: add correct result code to exception
	 */
	public function requestToken ()
	{
		OAuthRequestLogger::start($this);
		try
		{
			$this->verify(false);
			
			// Create a request token
			$store  = OAuthStore::instance();
			$token  = $store->addConsumerRequestToken($this->getParam('oauth_consumer_key', true));
		/**	$result = "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n" . '<request_token>oauth_token='.$this->urlencode($token['token'])
					.'&oauth_token_secret='.$this->urlencode($token['token_secret']) . '</request_token>';
					
			header('HTTP/1.1 200 OK');
			header('Content-Length: '.strlen($result));
		//	header('Content-Type: application/x-www-form-urlencoded');
			echo $result;*/
		/**	$data = array();
			$data['oauth_token'] = $token['token'];
			$data['oauth_token_secret'] = $token['token_secret'];
			$result = array();
			$result['success'] = 'true';
			$result['data'] = $data;
			print_r(json_encode($result));*/
			$this->redirect($token['callback_uri'], array('oauth_request_token'=>rawurlencode($token['token']),'oauth_request_token_secret'=>rawurlencode($token['token_secret'])));
			
		}
		catch (OAuthException $e)
		{
			$result = array();
			$result['success'] = 'false';
			print_r(json_encode($result));
		}

		OAuthRequestLogger::flush();
		exit();
	}
	
	public function deauthToken ()
	{
		OAuthRequestLogger::start($this);
		try
		{
			$userid = $this->verify('access');
			if (isset($userid)){
				// delete the access token
				$store  = OAuthStore::instance();
				$store->deleteConsumerAccessToken($this->getParam('oauth_token', true), $userid);

				$result = array();
				$result['success'] = 'true';
				print_r(json_encode($result));
			} else {
				$result = array();
				$result['success'] = 'true';
				print_r(json_encode($result));
			}
		}
		catch (OAuthException $e)
		{
			$result = array();
			$result['success'] = 'false';
			print_r(json_encode($result));
		}

		OAuthRequestLogger::flush();
		exit();
	}
	
	/**
	 * Verify the start of an authorization request.  Verifies if the request token is valid.
	 * 
	 * @exception OAuthException thrown when not a valid request
	 * @return array token description
	 */
	public function authorizeVerify ( )
	{
		OAuthRequestLogger::start($this);

		$store = OAuthStore::instance();
		$token = $this->getParam('oauth_token', true);
		$rs    = $store->getConsumerRequestToken($token);
		if (empty($rs))
		{
			throw new OAuthException('Unknown request token "'.$token.'"');
		}

		// We need to remember the callback		
		if (	empty($_SESSION['verify_oauth_token'])
			||	strcmp($_SESSION['verify_oauth_token'], $rs['token']))
		{
			$_SESSION['verify_oauth_token'] 		= $rs['token'];
			$_SESSION['verify_oauth_consumer_key']	= $rs['consumer_key'];
			$_SESSION['verify_oauth_callback']		= $rs['oauth_callback'];
		}

		OAuthRequestLogger::flush();
		return $rs;
	}
	
	
	/**
	 * Overrule this method when you want to want to display a nice page when
	 * the authorization is finished.  This function does not know if the authorization was
	 * succesfull, you need to check the token in the database.
	 */
	public function authorizeFinish ( $authorized, $user_id )
	{
		OAuthRequestLogger::start($this);

		$token = $this->getParam('oauth_token', true);
		if (	isset($_SESSION['verify_oauth_token']) 
			&&	$_SESSION['verify_oauth_token'] == $token)
		{
			// Flag the token as authorized, or remove the token when not authorized
			$store = OAuthStore::instance();
			
			if ($authorized)
			{
				OAuthRequestLogger::addNote('Authorized token "'.$token.'" for user '.$user_id);
				$store->authorizeConsumerRequestToken($token, $user_id);
			}
			else
			{
				OAuthRequestLogger::addNote('Authorization rejected for token "'.$token.'" for user '.$user_id."\nToken has been deleted");
				$store->deleteConsumerRequestToken($token);
			}
			
			if (!empty($_SESSION['verify_oauth_callback']))
			{
				$this->redirect($_SESSION['verify_oauth_callback'], array('oauth_token'=>rawurlencode($token)));
			}
		}
		OAuthRequestLogger::finish();
	}
	
	
	/**
	 * Exchange a request token for an access token.
	 * The exchange is only succesful if the request token has been authorized.
	 */
	public function accessToken ()
	{
		OAuthRequestLogger::start($this);

		try
		{
			$this->verify('request');
			
			$store  = OAuthStore::instance();
			
			//Delete other access tokens having same osr_requester_name && osr_usa_id_ref && osr_application_title
			$store->deletePreviousAccessToken($this->getParam('oauth_token', true));
			
			//Exchange
			$token  = $store->exchangeConsumerRequestForAccessToken($this->getParam('oauth_token', true));
			/**$result = "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n" . '<access_token>oauth_token='.$this->urlencode($token['token'])
					.'&oauth_token_secret='.$this->urlencode($token['token_secret']) . '</access_token>';
					
			header('HTTP/1.1 200 OK');
			header('Content-Length: '.strlen($result));
			//header('Content-Type: application/x-www-form-urlencoded');

			echo $result;*/
			
			/**$data = array();
			$data['oauth_token'] = $token['token'];
			$data['oauth_token_secret'] = $token['token_secret'];
			$result = array();
			$result['success'] = 'true';
			$result['data'] = $data;
			print_r(json_encode($result));*/
			$this->redirect($token['callback_uri'], array('oauth_access_token'=>rawurlencode($token['token']),'oauth_access_token_secret'=>rawurlencode($token['token_secret']),'user_name'=>rawurlencode($token['user_name'])));
		}
		catch (OAuthException $e)
		{
			header('HTTP/1.1 401 Access Denied');
			header('Content-Type: text/xml');
			throw new OAuthException($e->getMessage());
		}
		OAuthRequestLogger::flush();
		exit();
	}	
}

?>